//=============================================================================
//
//  CLASS Checkerboard
//
//=============================================================================
#ifndef RAYTRACER_CHECKERBOARD_H
#define RAYTRACER_CHECKERBOARD_H

//== INCLUDES =================================================================
#include "Surface.h"
#include "Plane.h"

//== CLASS DEFINITION =========================================================

/*!
	A plane 
*/
class Checkerboard : public Plane
{

    //-------------------------------------------------------------- private data
private:
    VectorType3 m_v3X, m_v3Y;
	int	m_xRaster;
	int m_yRaster;
	int m_activeShader;
	const Shader    *m_shader2;

    //-------------------------------------------------------------- private methods
private:

public:

    //-------------------------------------------------------------- constructors

    //! standard constructor
    Checkerboard() :
	m_v3X( 0.0 ),
	m_v3Y( 0.0 )
    {
		m_v3Normal = VectorType3( 0.0 );
		m_v3Point = VectorType3( 0.0 );
		m_activeShader = 1;
		m_xRaster = 10;
		m_yRaster = 10;
    };

    //! initializing constructor
    Checkerboard(
        const VectorType3   &v3Point,
		const VectorType3   &v3X,
		const VectorType3   &v3Y,
		int xRaster, int yRaster ) 
    {
		m_v3Point = VectorType3( v3Point );
		setXVector( v3X );
		setYVector( v3Y );
		m_activeShader = 1;
		m_xRaster = xRaster;
		m_yRaster = yRaster;
		m_v3Normal = (m_v3X-m_v3Point)%(m_v3Y-m_v3Point); 
    };

    //-------------------------------------------------------------- element access

	   //-------------------------------------------------------------- element access
    virtual void setShader2(
        const Shader    *pclShader )    
    {
        m_shader2 = pclShader;
    };

    
    virtual const Shader *getShader() const
    {
		if ( m_activeShader == 1 )
			return m_shader;
		else
			return m_shader2;
    };


    
    void setXVector(
        const VectorType3   &v3X )  
    {
        m_v3X = v3X;
    };

    
    const VectorType3 &getXVector() const
    {
        return m_v3X;
    };

    
    void setYVector(
        const VectorType3   &v3Y )  
    {
        m_v3Y = v3Y;
    };

    
    const VectorType3 &getYVector() const
    {
        return m_v3Y;
    };

    void setXRaster( int xRaster )  
    {
        m_xRaster = xRaster;
    };

    
    int getXRaster() const
    {
        return m_xRaster;
    };
    void setYRaster( int yRaster )  
    {
        m_yRaster = yRaster;
    };

    
    int getYRaster() const
    {
        return m_yRaster;
    };

	
void getRotation(
    int *rotx,  
    int *roty,  
    int *rotz ); 
//rotate object
void rotate(
    float   rotx,   
    float   roty,   
    float   rotz );  

    //render object in OpenGL
    void    renderGL() const;

    //--------------------------------------------------------------- intersections
    bool    intersect( const Ray       &clRay,
                       const double  t0,
                       const double  t1,
                       TracingContext  &tContext ) const;
};

//=============================================================================
#endif // RAYTRACER_CHECKERBOARD_H

