//=============================================================================
//
//  CLASS Ray
//
//=============================================================================
#ifndef RAYTRACER_RAY_H
#define RAYTRACER_RAY_H

//== INCLUDES =================================================================
#include "Types.h"


//== CLASS DEFINITION =========================================================

/*!
	Simple ray: A point and a direction.
*/
class   Ray
{

    //-------------------------------------------------------------- private data
private:
    VectorType3 m_v3Origin;
    VectorType3 m_v3Dir;
    ColorType   m_colPower;

    //-------------------------------------------------------------- private methods
private:

public:

    //-------------------------------------------------------------- constructors

    //! standard constructor
    Ray() :
    m_v3Origin( 0, 0, 0 ),
    m_v3Dir( 0, 0, 0 ),
    m_colPower( 0.0 )
    {
    };

    //! Constructor with initializing values
    Ray(
        const VectorType3   origin, 
        const VectorType3   dir,    
        const ColorType     power )  :
    m_v3Origin( origin ),
    m_v3Dir( dir ),
    m_colPower( power )
    {
    };

    //! destructor	
    ~Ray()
    {
    };

    //----------------------------------------------------------- element access

    //! set ray origin
    void setOrigin(
        const VectorType3   &origin )   
    {
        m_v3Origin = origin;
    };

    //! get ray origin
    const VectorType3 &getOrigin() const
    {
        return m_v3Origin;
    };

    //! set ray direction
    void setDir(
        const VectorType3   &dir )  
    {
        m_v3Dir = dir;
    };

    //! get ray direction
    const VectorType3 &getDir() const
    {
        return m_v3Dir;
    };

    //! set ray power (color)
    void setPower(
        const ColorType &power )    
    {
        m_colPower = power;
    };

    //! get ray power (color)
    const ColorType &getPower() const
    {
        return m_colPower;
    };

    //----------------------------------------------------------- operators
    //----------------------------------------------------------- functionality

    //!	evaluate ray at parameter t
    VectorType3 evaluate(
        double    t )  const
    {
        return m_v3Origin + t * m_v3Dir;
    };

    //! Reflect ray at point around normal.
    Ray     reflectedRay( const VectorType3   &v3Point,
                          const VectorType3   &v3Normal ) const;

    //!	Refract ray at point given normal and refractive index.
    bool    refractedRay( const VectorType3   &v3Point,
                          const VectorType3   &v3Normal,
                          double            rRefractiveIndex,
                          Ray                 &clRefractedRay ) const;
};

//=============================================================================
#endif // RAYTRACER_RAY_H

